/*     BIBLIOTHEQUE 

Le module gre les fonctions associes aux auteurs
	. liste des auteurs
	. ajout d'un auteur
	. modification d'un auteur
	. validation de l'ajout/modif
	. suppression d'un auteur
	. validation de la suppression

Le passage de donnes entre le controleur et la vue s'effectue par l'intermdiaire du champ zechange qui fonctionne ainsi:
	. titre   = titre de la page spcifique
	. domaine = domaine pour l'integration des scripts
	. ligne   = objet reprsentant une ligne de la base + le message d'erreur
	. liste   = tableau contenant toutes les lignes  afficher
Il participe  l'change entre fonctions et entre les fonctions et les vues.
Ce champ est cr par la fonction generale init_zechange.

DJ 22.11.2014
*/


// Modules
var omep=require('X:/.../controleurs/presentation');
var odb=require("X:/.../basedonnees");
var ogen=require('X:/.../controleurs/general');

// exportation des fonctions
exports.liste = liste;
exports.ajout = ajout;
exports.modif = modif;
exports.ajoutvalid = ajoutvalid;
exports.supprvalid = supprvalid;


// domaine concern pour personnaliser notamment les scripts intgrs dans la page finale
var domaine = "auteurs";



/*-------------------------------------------------------------------------------------------------
LISTE DES AUTEURS
-------------------------------------------------------------------------------------------------*/

function liste(request, response) {

	/* Description de la fonction 
	Le coeur du traitement est la fonction db.each qui reoit 3 paramtres :
		. la requte sql
		. la fonction de callback pour chaque ligne lue (concatnation avec les lignes prcdentes)
		. la fonction de callback quand toute la requte est lue (criture de la page)

	Elle est srialise pour s'assurer de recevoir les donnes dans l'ordre voulu.

	*/
	
	var zechange=ogen.init_zechange("LISTE DES AUTEURS", domaine);

	var db=odb.ouverture_base();
	db.serialize(function() {
  
	// lecture de la base de donnes et stockage dans un tableau
	var req="SELECT a.id, a.autabr, a.autnom, ifnull(l.nbre, 0) as nbre from auteurs a ";
	req += "left join (select auteur_id, count(*) as nbre ";
	req += "from livres ";
	req += "group by auteur_id) l on l.auteur_id = a.id ";
	req += "order by autabr";
	var i = 0;

	db.each(req, 
		
		// cette fonction de callback traite chaque ligne lue
		function(err, row) {
			zechange.liste[i]=row;
			i++;
			} 

		,
		// cette fonction de callback est active lorsque le traitement de la requete est termin
		// err=message d'erreur     nbr=nombre de lignes lues
		function(err, nbr) {
		
			// intgration dans la page complte et affichage
			omep.afficher_page(request, response, c_auteurs_vue_liste, zechange);
			}

		);
  

	});

}



/*-------------------------------------------------------------------------------------------------
AJOUT D'UN AUTEUR - AFFICHAGE FORMULAIRE
-------------------------------------------------------------------------------------------------*/

function ajout(req, response) {
	console.log("Execution de l'ajout d'un auteur.");

	var zechange=ogen.init_zechange("AJOUT D'UN AUTEUR", domaine);
	zechange.ligne.id=0; // indique une cration

	// afficher le formulaire avec l'objet cr
	omep.afficher_page(req, response, c_auteurs_vue_modif, zechange);

}



/*-------------------------------------------------------------------------------------------------
MODIFICATION D'UN AUTEUR - AFFICHAGE FORMULAIRE
-------------------------------------------------------------------------------------------------*/

function modif(request, response) {
	console.log("Execution de la modification d'un auteur.");

	// Rcupration des variables de l'url (id  modifier)
	var pid=request.query.id;  //avec la syntaxe /modif?id=xxx
	var zechange=ogen.init_zechange("MODIFICATION D'UN AUTEUR", domaine);

	var db=odb.ouverture_base();
	db.serialize(function() {
  
	// lecture de la base de donnes pour alimenter l'objet
	var req="SELECT a.id, a.autabr, a.autnom ";
	req += "from auteurs a ";
	req += "where a.id = " + pid;

	db.each(req, 
		
		// cette fonction de callback traite chaque ligne lue (1 seule ici)
		function(err, row) {
			// renseigner l'objet avec les valeurs lues
			zechange.ligne=row;
			} 

		,
		// cette fonction de callback est active lorsque le traitement de la requete est termin
		function() {
			// afficher le formulaire de saisie pour l'objet
			omep.afficher_page(request, response, c_auteurs_vue_modif, zechange);
			}

		);
  

	});

}

/*-------------------------------------------------------------------------------------------------
VALIDATION DE L'AJOUT/MODIF D'UN AUTEUR
	. lecture des donnes du formulaire
	. contrle
	. maj de la base
-------------------------------------------------------------------------------------------------*/

function ajoutvalid(request, response) {

	console.log("validation de l'ajout/modif d'un auteur");

	//-----  Lecture du formulaire

	if (request.method != 'POST') {
		return;
	}

	var zechange=ogen.init_zechange("MODIFICATION D'UN AUTEUR", domaine);
	zechange.ligne.id=request.body.zid;
	zechange.ligne.autnom=request.body.zautnom;
	zechange.ligne.autabr=request.body.zautabr;

	//-----  Controle des donnes
	if (zechange.ligne.id == 0) {
		zechange.titre = "AJOUT AUTEUR APRES ERREUR";
		}
	else {
		zechange.titre = "MODIF AUTEUR APRES ERREUR";
		}

	zechange.ligne.meserr="";
	var inderr = 0;

	// nom obligatoire
	if (inderr == 0 && zechange.ligne.autnom=="") {
		inderr=1;
		zechange.ligne.meserr="Le nom est obligatoire";
		}
	

	// nom court obligatoire
	if (inderr == 0 && zechange.ligne.autabr=="") {
		inderr=1;
		zechange.ligne.meserr="Le nom court est obligatoire";
		}
		
	// si erreur, afficher le message au bas de l'cran
	if (inderr == 1) {
			omep.afficher_page(request, response, c_auteurs_vue_modif, zechange);
			return;
			}
		
//-----  Maj de la base
		// ... calcul requete
		var sql="";
		
		if (zechange.ligne.id == 0) {
			// ajout d'un auteur
			sql = "insert into auteurs (autnom, autabr) values('"
			sql += zechange.ligne.autnom;
			sql += "', '";
			sql += zechange.ligne.autabr;
			sql += "')";
			}
			else
			{
			// modification d'un auteur
			sql = "update auteurs set "
			sql += "autnom = '" + zechange.ligne.autnom + "'";
			sql += ", ";
			sql += "autabr = '" + zechange.ligne.autabr + "'";
			sql += " where id=" + zechange.ligne.id;
			}

		// ... execution requete puis retour  la liste
		odb.maj_base(request, response, sql, c_auteurs_url_liste);
		
}



/*-------------------------------------------------------------------------------------------------
VALIDATION DE LA SUPPRESSION D'UN AUTEUR
	. lecture de l'id  supprimer dans l'url
	. maj de la base
-------------------------------------------------------------------------------------------------*/

function supprvalid(request, response) {

	console.log("validation de la suppression d'un auteur");

	// Rcupration des variables de l'url (id  modifier)
	var params = request.query;
	var pid = params.id;

	//-----  Maj de la base
	// ... calcul requete
	var sql="";
	sql = "delete from auteurs where id=" + pid;

	// ... execution requete
	odb.maj_base(request, response, sql, c_auteurs_url_liste);
}
